module commondata
 implicit none
 integer, parameter :: nx=nnx, ny=nny, nz=nnz
 integer, parameter :: nycpu=nnycpu, nzcpu=nnzcpu
 integer :: rank, ntask
! optional parameter for MPI subroutine with USE MPI_F08, included so that the code is retrocompatible with USE MPI
 integer :: ierr
! for use mpi_f08
!type(mpi_comm) :: cart_comm
 integer :: cart_comm
 double precision :: xl,yl
 character(len=50) :: folder='./results'
end module commondata



module grid
 use commondata
 double precision :: x(nx),y(ny),z(nz)
end module grid



module velocity
 double precision, allocatable :: u(:,:,:), v(:,:,:), w(:,:,:)
 double precision, allocatable :: uc(:,:,:,:), vc(:,:,:,:), wc(:,:,:,:)
 double precision, allocatable :: wa2(:,:,:), wa3(:,:,:)
 double precision, allocatable, dimension(:,:,:,:) :: sgradpx,sgradpy
end module velocity



module wavenumber
 use commondata
 double precision :: kx(nx/2+1),ky(ny),k2(nx/2+1,ny)
end module wavenumber



module sim_par
 double precision, parameter :: pi=3.14159265358979
 double precision :: Re, dt, gradpx, gradpy, Co
 double precision :: gamma
! b.c. on w and omega for Helmholtz problem
 double precision, dimension(2) :: p_u,q_u,r_u,p_v,q_v,r_v,p_w,q_w,r_w,p_o,q_o,r_o,zp
 integer :: bc_up, bc_low
 integer :: restart, nt_restart, in_cond, dump_failure
 integer :: nstart, nend, ndump,sdump
end module sim_par



module phase_field
 integer :: phi_flag,in_cond_phi,b_type
 double precision :: rhor,visr,We,Ch,Pe,Fr,grav(3),s_coeff
 double precision, allocatable :: phi(:,:,:), phic(:,:,:,:),one_s(:,:,:,:)
end module phase_field



module surfactant
! mean surfactant concentration, initial condition parameters, ...
 integer :: psi_flag, in_cond_psi
 double precision :: Ex, Pe_psi, P_i, El
 double precision, allocatable :: psi(:,:,:),psic(:,:,:,:)
end module surfactant



module temperature
 integer :: theta_flag,in_cond_theta
 double precision :: Ra,Pr
 double precision, dimension(2) :: p_theta,q_theta,r_theta
 double precision, allocatable :: theta(:,:,:),thetac(:,:,:,:)
end module temperature



module velocity_old
 ! store velocities at t=n-1, needed for non-linear part of time derivatives for non-matched densities case
 double precision, allocatable :: ucp(:,:,:,:), vcp(:,:,:,:), wcp(:,:,:,:)
end module velocity_old



module mpiIO
 integer :: ftype,stype
end module mpiIo



module par_size
! size in physical space: nx,npz,npy
 integer :: fpy,fpz
! size in spectral space:
 integer :: spx,spy
 integer :: cstart(3),fstart(3)
end module par_size



module fftw3
 use, intrinsic :: iso_c_binding
 include 'fftw3.f03'
 type(c_ptr) :: plan_x_fwd,plan_y_fwd,plan_z_fwd
 type(c_ptr) :: plan_x_bwd,plan_y_bwd,plan_z_bwd
end module fftw3



module sterms
 double precision, allocatable, dimension(:,:,:,:) :: s1_o,s2_o,s3_o, sphi_o, spsi_o, stheta_o
end module sterms



module stats
 integer :: flowiter,stat_dump,stat_start
 integer :: plane_comm,col_comm
end module stats
